\\ Copyright 2021, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(strictargs,1);
default(recover,0);
read("../vecper.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");

logbase(b,x)=log(x)/log(b)
log2(x)=logbase(2,x)
log3(x)=logbase(3,x)


real_to_frac(f) =
{
  if(type(f)=="t_REAL",
     \\ binary() gives [ [bits above point], [bits below point] ]
     \\ or 0.0 is just [0,...,0]
     my(v=binary(f));
     if(f,subst(Pol(v[2]),'x,2)) \\ like fromdigits(), but work in gp 2.7.x
     / 2^#v[2];
     ,
     f);
}

take_ternary_digit(f) =
{
  (0 <= f && f <= 1) || error("oops, f out of range: ",f);
  f*=3;
  my(d=min(2,floor(f)));
  [f-d, d];
}



\\-----------------------------------------------------------------------------
R = [ [0,0], [1,0], [1,1] ]

z(t) =
{
  my(x=0,y=0);
  for(i=1,10,
     [t,d]=take_ternary_digit(t);
     x += R[d+1][1]/3^i;
     y += R[d+1][2]/2^i);
  [x,y];
}
check_equal(take_ternary_digit(1), [1,2]);
z(1)
check_equal(z(0), [0,0], "z(0)");
check_equal(z(1), [(1-1/3^10)/2, 1-1/2^10], "z(1)");

R = [ [0,0], [1,1], [1,0] ]
k=8;
v=apply(z,[0..3^k-1]/3^k);
plothraw(apply(e->e[1],v), apply(e->e[2],v))
quit


\\-----------------------------------------------------------------------------
\\ carpet

takedigit(x) = x*=3; [frac(x),floor(x)];
f(x) = sum(k=1,20,[x,d]=takedigit(x); (d!=1)/2^k));
ploth(x=0,1,f(x))

h(x) = my(s=1); sum(k=1,20,[x,d]=takedigit(x); s=bitxor(s,d==1); s/2^k);
ploth(x=0,1,h(x))


A094114(n) = n--; my(v=digits(n,3),s=-1); \
  for(i=1,#v, v[i]=if(v[i]%2, s=-s)); fromdigits(v,2);
OEIS_check_func("A094114");

A332497(n) = my(v=digits(n,3),s=0); \
  for(i=1,#v, s=bitxor(s,v[i]==1); v[i]=s); fromdigits(v,2);
OEIS_check_func("A332497");

A332498(n) = my(v=digits(n,3),s=0); \
  for(i=1,#v, s=bitxor(s,v[i]==2); v[i]=s); fromdigits(v,2);
OEIS_check_func("A332497");

A289813(n) = fromdigits(digits(n,3)%2, 2);
OEIS_check_func("A289813");

OEIS_data("A332497")[1..27] - \
OEIS_data("A094114")[1..27]
quit

\\-----------------------------------------------------------------------------
\\ metric and Hausdorff dimensions

\\ n=3;     \\ width
\\ m=2;     \\ height
\\ r=3;     \\ num blocks filled
\\ s=2;     \\ num rows with one or more blocks
\\ log(s)/log(m) + log(r/s)/log(n)
\\ log2(1 + 2^(log3(2)))

\\ n=3;     \\ width
\\ m=3;     \\ height
\\ r=5;     \\ num blocks filled
\\ s=3;     \\ num rows with one or more blocks
\\ log(s)/log(m) + log(r/s)/log(n)
\\ log2(1 + 2^(log3(2)))

M = [1,0,1; 0,1,0; 1,0,1]
M = [1,0,1; 0,1,0]

R(i,j) = M[j,i];
[n,m] = Vecrev(matsize(M))
print("size n="n" m="m);
r = sum(i=1,n, sum(j=1,m, R(i,j)));
s = sum(j=1,m, sum(i=1,n,R(i,j))!=0);
print("r = "r" card");
print("s = "s" columns not empty");
metric_dim = ( logbase(m,s) + logbase(n, r/s) )
t(j) = sum(i=1,n, R(i,j));       \\ how many i with (i,j) in R, num in column
print("t: ",vector(m,j, t(j)))
Hausdorff_dim = logbase(m, sum(j=1,m, t(j)^logbase(n,m)) )
logbase(2, 1+2^logbase(3,2))
quit

\\ 5 star pattern = log(5)/log(3)  A113209      log_n(r)
\\ 3 of 6 pattern
\\ not in OEIS: 1.3690702464
\\ not in OEIS: 1.3496838201

\\-----------------------------------------------------------------------------
\\ without flip
\\ R = { (0,0, (1,1), (2,0) }
\\ odd digit push up
\\
\\ ternary to binary 1 at odd
\\ vector(50,n, fromdigits(digits(n,3)%2,2))
\\ A289813
carpet_f_to_vecper(f) =
{
  f=real_to_frac(f);

  \\ "state" in a 2-element vector [f_remaining, sign +1 or -1].
  vecper_make_state
    ((f)->
     my(d); [f,d]=take_ternary_digit(f);
     [d%2, f];
     ,
     f);   \\ initial whole f, sign +1
}
Hironaka_f_to_y(f) = vecper_frac_fromdigits(carpet_f_to_vecper(f), 2);

\\ 0 = .000 and 1 = .222 both become binary .000
carpet_f_to_vecper(0)   == [[], [0]]   || error();
carpet_f_to_vecper(1)   == [[], [0]]   || error();
Hironaka_f_to_y(0)   == 0                || error();
Hironaka_f_to_y(1)   == 0                || error();

\\ 1/2 = .111 becomes .111 unchanged
carpet_f_to_vecper(1/2) == [[], [1]]   || error();
Hironaka_f_to_y(1/2) == 1                || error();


{
  my(len=3^5,
     \\ f values 0/len, 1/len, ..., len/len
     f_vec=vector(len+1,i, (i-1)/len),
     y_vec=apply(Hironaka_f_to_y, f_vec));
\\ print(f_vec);
\\ print(y_vec);
  print("y range "vecmin(y_vec)" .. "vecmax(y_vec));
  plothraw(f_vec,y_vec, 1);
}
quit



\\-----------------------------------------------------------------------------
\\ A094114  - Sum_{i=1..n-1} (-1)^i*2^valuation(i,3).
\\     1, 0, 2, 1, 2, 0, 1, 0, 4, 3, 4, 2, 3, 2, 4, 3, 4, 0, 1, 0, 2, 1, 2
\\  0, 1, 0,  2, 1, 2,  0, 1, 0,   4, 3, 4, 2, 3, 2, 4, 3, 4, 0, 1, 0, 2, 1, 2,
a(n) =
{
  my(v=digits(n,3), s=1);
  for(i=1,#v, if(v[i]%=2, v[i]=s;s=-s));
  fromdigits(v,2);
}
vector(30,n,a(n))


print("1+log3(2)");
1+log3(2)     \\ A153459 Decimal expansion of log_3 (6).
my(v=OEIS_data("A153459")); fromdigits(v)/10^(#v-1) *1.0
\\ localprec(2000); \
\\ digits(floor((1+log3(2))*10^999)) == OEIS_bfile_samples("A153459")

2^64
exp(64 * log(2))

print("2^log3(2)");
2^log3(2)
\\ not in OEIS: 1.54856265263024290

print("log2(1+2^log3(2))");
log2(1+2^log3(2))
\\ not in OEIS: 1.34968382019
log(1 + 2^(log(2)/log(3)))/log(2)
log(1 + exp(log(2)^2/log(3)) )/log(2)


\\-----------------------------------------------------------------------------
